@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2014 IBM Corporation and others.
+ * Copyright (c) 2000, 2015 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -9,6 +9,7 @@
  *     IBM Corporation - initial API and implementation
  *     Stephan Herrmann - Contribution for
  *								Bug 438458 - [1.8][null] clean up handling of null type annotations wrt type variables
+ *								Bug 429813 - [1.8][dom ast] IMethodBinding#getJavaElement() should return IMethod for lambda
  *******************************************************************************/
 
 package org.aspectj.org.eclipse.jdt.core.dom;
@@ -24,8 +25,7 @@ import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.BaseTypeBinding;
 import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.Binding;
 import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.CaptureBinding;
 import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.FieldBinding;
-import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.IntersectionCastTypeBinding;
-import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.LocalTypeBinding;
+import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.IntersectionTypeBinding18;
 import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.MethodBinding;
 import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.PackageBinding;
 import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.ParameterizedTypeBinding;
@@ -60,7 +60,7 @@ class TypeBinding implements ITypeBinding {
 	org.aspectj.org.eclipse.jdt.internal.compiler.lookup.TypeBinding binding;
 	private TypeBinding prototype = null;
 	private String key;
-	private BindingResolver resolver;
+	protected BindingResolver resolver;
 	private IVariableBinding[] fields;
 	private IAnnotationBinding[] annotations;
 	private IAnnotationBinding[] typeAnnotations;
@@ -71,6 +71,18 @@ class TypeBinding implements ITypeBinding {
 	private ITypeBinding[] bounds;
 	private ITypeBinding[] typeParameters;
 
+	/**
+	 * Create either a regular TypeBinding or an AnonymousTypeBinding (if declaringMember is given).
+	 */
+	public static TypeBinding createTypeBinding(BindingResolver resolver,
+												org.aspectj.org.eclipse.jdt.internal.compiler.lookup.TypeBinding referenceBinding,
+												IBinding declaringMember)
+	{
+		return declaringMember != null
+					? new LocalTypeBinding(resolver, referenceBinding, declaringMember)
+					: new TypeBinding(resolver, referenceBinding);
+	}
+
 	public TypeBinding(BindingResolver resolver, org.aspectj.org.eclipse.jdt.internal.compiler.lookup.TypeBinding binding) {
 		this.binding = binding;
 		this.resolver = resolver;
@@ -369,8 +381,8 @@ class TypeBinding implements ITypeBinding {
 	 * @see ITypeBinding#getDeclaringMethod()
 	 */
 	public synchronized IMethodBinding getDeclaringMethod() {
-		if (this.binding instanceof LocalTypeBinding) {
-			LocalTypeBinding localTypeBinding = (LocalTypeBinding) this.binding;
+		if (this.binding instanceof org.aspectj.org.eclipse.jdt.internal.compiler.lookup.LocalTypeBinding) {
+			org.aspectj.org.eclipse.jdt.internal.compiler.lookup.LocalTypeBinding localTypeBinding = (org.aspectj.org.eclipse.jdt.internal.compiler.lookup.LocalTypeBinding) this.binding;
 			MethodBinding methodBinding = localTypeBinding.enclosingMethod;
 			if (methodBinding != null) {
 				try {
@@ -440,6 +452,11 @@ class TypeBinding implements ITypeBinding {
 		return null;
 	}
 
+	@Override
+	public IBinding getDeclaringMember() {
+		return null;
+	}
+
 	/*
 	 * @see ITypeBinding#getDimensions()
 	 */
@@ -468,7 +485,7 @@ class TypeBinding implements ITypeBinding {
 	public ITypeBinding getTypeDeclaration() {
 		if (this.binding instanceof ParameterizedTypeBinding)
 			return this.resolver.getTypeBinding(((ParameterizedTypeBinding)this.binding).genericType());
-		return this.resolver.getTypeBinding(this.binding.unannotated(false));
+		return this.resolver.getTypeBinding(this.binding.unannotated());
 	}
 	
 	/* (non-Javadoc)
@@ -685,9 +702,9 @@ class TypeBinding implements ITypeBinding {
 				buffer.append(brackets);
 				return String.valueOf(buffer);
 
-			case Binding.INTERSECTION_CAST_TYPE :
-				// just use the first bound for now (same kludge as in IntersectionCastTypeBinding#constantPoolName())
-				return new String(((IntersectionCastTypeBinding) this.binding).getIntersectingTypes()[0].sourceName());
+			case Binding.INTERSECTION_TYPE18 :
+				// just use the first bound for now (same kludge as in IntersectionTypeBinding18#constantPoolName())
+				return new String(((IntersectionTypeBinding18) this.binding).getIntersectingTypes()[0].sourceName());
 
 			default :
 				if (isPrimitive() || isNullType()) {
@@ -711,7 +728,7 @@ class TypeBinding implements ITypeBinding {
 			case Binding.TYPE_PARAMETER : // includes capture scenario
 			case Binding.WILDCARD_TYPE :
 			case Binding.INTERSECTION_TYPE:
-			case Binding.INTERSECTION_CAST_TYPE:
+			case Binding.INTERSECTION_TYPE18:
 				return null;
 		}
 		ReferenceBinding referenceBinding = (ReferenceBinding) this.binding;
@@ -808,7 +825,7 @@ class TypeBinding implements ITypeBinding {
 				}
 				return String.valueOf(buffer);
 			default :
-				if (isAnonymous() || this.binding.isLocalType() || this.binding.isIntersectionCastType()) {
+				if (isAnonymous() || this.binding.isLocalType() || this.binding.isIntersectionType18()) {
 					return NO_NAME;
 				}
 				if (isPrimitive() || isNullType()) {
@@ -1075,7 +1092,7 @@ class TypeBinding implements ITypeBinding {
 			if (!(type instanceof TypeBinding)) return false;
 			org.aspectj.org.eclipse.jdt.internal.compiler.lookup.TypeBinding expressionType = ((TypeBinding) type).binding;
 			// simulate capture in case checked binding did not properly get extracted from a reference
-			expressionType = expressionType.capture(scope, 0);
+			expressionType = expressionType.capture(scope, 0, 0);
 			return TypeBinding.EXPRESSION.checkCastTypesCompatibility(scope, this.binding, expressionType, null);
 		} catch (AbortCompilation e) {
 			// don't surface internal exception to clients
@@ -1132,7 +1149,7 @@ class TypeBinding implements ITypeBinding {
 			return false;
 		}
 		org.aspectj.org.eclipse.jdt.internal.compiler.lookup.TypeBinding otherBinding = ((TypeBinding) other).binding;
-		if (org.aspectj.org.eclipse.jdt.internal.compiler.lookup.TypeBinding.equalsEquals(otherBinding.unannotated(false), this.binding.unannotated(false))) {
+		if (org.aspectj.org.eclipse.jdt.internal.compiler.lookup.TypeBinding.equalsEquals(otherBinding.unannotated(), this.binding.unannotated())) {
 			return true;
 		}
 		// check return type
@@ -1345,4 +1362,22 @@ class TypeBinding implements ITypeBinding {
 		this.typeAnnotations = resolveAnnotationBindings(this.binding.getTypeAnnotations(), true);
 		return this.typeAnnotations;
 	}
+
+	static class LocalTypeBinding extends TypeBinding {
+
+		private IBinding declaringMember;
+
+		public LocalTypeBinding(BindingResolver resolver,
+									org.aspectj.org.eclipse.jdt.internal.compiler.lookup.TypeBinding binding,
+									IBinding declaringMember)
+		{
+			super(resolver, binding);
+			this.declaringMember = declaringMember;
+		}
+
+		@Override
+		public IBinding getDeclaringMember() {
+			return this.declaringMember;
+		}
+	}
 }
