@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2014 IBM Corporation and others.
+ * Copyright (c) 2000, 2015 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -10,10 +10,15 @@
  *     Stephan Herrmann - Contribution for
  *								Bug 400874 - [1.8][compiler] Inference infrastructure should evolve to meet JLS8 18.x (Part G of JSR335 spec)
  *								Bug 429384 - [1.8][null] implement conformance rules for null-annotated lower / upper type bounds
+ *								Bug 441797 - [1.8] synchronize type annotations on capture and its wildcard
+ *								Bug 456497 - [1.8][null] during inference nullness from target type is lost against weaker hint from applicability analysis
+ *								Bug 456924 - StackOverflowError during compilation
+ *								Bug 462790 - [null] NPE in Expression.computeConversion()
  *******************************************************************************/
 package org.aspectj.org.eclipse.jdt.internal.compiler.lookup;
 
 import org.aspectj.org.eclipse.jdt.core.compiler.CharOperation;
+import org.aspectj.org.eclipse.jdt.internal.compiler.ast.ASTNode;
 import org.aspectj.org.eclipse.jdt.internal.compiler.ast.Wildcard;
 import org.aspectj.org.eclipse.jdt.internal.compiler.classfmt.ClassFileConstants;
 import org.aspectj.org.eclipse.jdt.internal.compiler.impl.CompilerOptions;
@@ -26,30 +31,47 @@ public class CaptureBinding extends TypeVariableBinding {
 
 	/* information to compute unique binding key */
 	public ReferenceBinding sourceType;
-	public int position;
+	public int start;
+	public int end;
+	public ASTNode cud; // to facilitate recaptures.
 
-	public CaptureBinding(WildcardBinding wildcard, ReferenceBinding sourceType, int position, int captureID) {
-		super(TypeConstants.WILDCARD_CAPTURE_NAME_PREFIX, null, 0, wildcard.environment);
+	TypeBinding pendingSubstitute; // for substitution of recursive captures, see https://bugs.eclipse.org/456924
+
+	public CaptureBinding(WildcardBinding wildcard, ReferenceBinding sourceType, int start, int end, ASTNode cud, int captureID) {
+		super(TypeConstants.WILDCARD_CAPTURE_NAME_PREFIX, wildcard.environment);
 		this.wildcard = wildcard;
 		this.modifiers = ClassFileConstants.AccPublic | ExtraCompilerModifiers.AccGenericSignature; // treat capture as public
 		this.fPackage = wildcard.fPackage;
 		this.sourceType = sourceType;
-		this.position = position;
+		this.start = start;
+		this.end = end;
 		this.captureID = captureID;
 		this.tagBits |= TagBits.HasCapturedWildcard;
+		this.cud = cud;
 		if (wildcard.hasTypeAnnotations()) {
-			setTypeAnnotations(wildcard.getTypeAnnotations(), wildcard.environment.globalOptions.isAnnotationBasedNullAnalysisEnabled);
+			// register an unannoted version before adding the annotated wildcard:
+			CaptureBinding unannotated = (CaptureBinding) clone(null);
+			unannotated.wildcard = (WildcardBinding) this.wildcard.unannotated();
+			this.environment.getUnannotatedType(unannotated);
+			this.id = unannotated.id; // transfer fresh id
+			// now register this annotated type:
+			this.environment.typeSystem.cacheDerivedType(this, unannotated, this);
+			// propagate from wildcard to capture - use super version, because our own method propagates type annotations in the opposite direction:
+			super.setTypeAnnotations(wildcard.getTypeAnnotations(), wildcard.environment.globalOptions.isAnnotationBasedNullAnalysisEnabled);
 			if (wildcard.hasNullTypeAnnotations())
 				this.tagBits |= TagBits.HasNullTypeAnnotation;
+		} else {			
+			computeId(this.environment);
 		}
 	}
 	
 	// for subclass CaptureBinding18
-	protected CaptureBinding(ReferenceBinding sourceType, char[] sourceName, int position, int captureID, LookupEnvironment environment) {
+	protected CaptureBinding(ReferenceBinding sourceType, char[] sourceName, int start, int end, int captureID, LookupEnvironment environment) {
 		super(sourceName, null, 0, environment);
 		this.modifiers = ClassFileConstants.AccPublic | ExtraCompilerModifiers.AccGenericSignature; // treat capture as public
 		this.sourceType = sourceType;
-		this.position = position;
+		this.start = start;
+		this.end = end;
 		this.captureID = captureID;
 	}
 
@@ -57,10 +79,12 @@ public class CaptureBinding extends TypeVariableBinding {
 		super(prototype);
 		this.wildcard = prototype.wildcard;
 		this.sourceType = prototype.sourceType;
-		this.position = prototype.position;
+		this.start = prototype.start;
+		this.end = prototype.end;
 		this.captureID = prototype.captureID;
 		this.lowerBound = prototype.lowerBound;
 		this.tagBits |= (prototype.tagBits & TagBits.HasCapturedWildcard);
+		this.cud = prototype.cud;
 	}
 	
 	// Captures may get cloned and annotated during type inference.
@@ -81,7 +105,7 @@ public class CaptureBinding extends TypeVariableBinding {
 		}
 		buffer.append(TypeConstants.WILDCARD_CAPTURE);
 		buffer.append(this.wildcard.computeUniqueKey(false/*not a leaf*/));
-		buffer.append(this.position);
+		buffer.append(this.end);
 		buffer.append(';');
 		int length = buffer.length();
 		char[] uniqueKey = new char[length];
@@ -128,7 +152,7 @@ public class CaptureBinding extends TypeVariableBinding {
 			switch (this.wildcard.boundKind) {
 				case Wildcard.EXTENDS :
 					// still need to capture bound supertype as well so as not to expose wildcards to the outside (111208)
-					TypeBinding capturedWildcardBound = originalWildcardBound.capture(scope, this.position);
+					TypeBinding capturedWildcardBound = originalWildcardBound.capture(scope, this.start, this.end);
 					if (originalWildcardBound.isInterface()) {
 						this.setSuperClass(scope.getJavaLangObject());
 						this.setSuperInterfaces(new ReferenceBinding[] { (ReferenceBinding) capturedWildcardBound });
@@ -180,7 +204,7 @@ public class CaptureBinding extends TypeVariableBinding {
 		switch (this.wildcard.boundKind) {
 			case Wildcard.EXTENDS :
 				// still need to capture bound supertype as well so as not to expose wildcards to the outside (111208)
-				TypeBinding capturedWildcardBound = originalWildcardBound.capture(scope, this.position);
+				TypeBinding capturedWildcardBound = originalWildcardBound.capture(scope, this.start, this.end);
 				if (originalWildcardBound.isInterface()) {
 					this.setSuperClass(substitutedVariableSuperclass);
 					// merge wildcard bound into variable superinterfaces using glb
@@ -253,6 +277,15 @@ public class CaptureBinding extends TypeVariableBinding {
 		return false;
 	}
 
+	@Override
+	public boolean isProperType(boolean admitCapture18) {
+		if (this.lowerBound != null && !this.lowerBound.isProperType(admitCapture18))
+			return false;
+		if (this.wildcard != null && !this.wildcard.isProperType(admitCapture18))
+			return false;
+		return super.isProperType(admitCapture18);
+	}
+
 	public char[] readableName() {
 		if (this.wildcard != null) {
 			StringBuffer buffer = new StringBuffer(10);
@@ -268,6 +301,20 @@ public class CaptureBinding extends TypeVariableBinding {
 		}
 		return super.readableName();
 	}
+	
+	public char[] signableName() {
+		if (this.wildcard != null) {
+			StringBuffer buffer = new StringBuffer(10);
+			buffer
+				.append(TypeConstants.WILDCARD_CAPTURE_SIGNABLE_NAME_SUFFIX)
+				.append(this.wildcard.readableName());
+			int length = buffer.length();
+			char[] name = new char[length];
+			buffer.getChars(0, length, name, 0);
+			return name;
+		}
+		return super.readableName();
+	}
 
 	public char[] shortReadableName() {
 		if (this.wildcard != null) {
@@ -295,7 +342,7 @@ public class CaptureBinding extends TypeVariableBinding {
 			try {
 				if (this.wildcard != null) {
 					nameBuffer.append("of "); //$NON-NLS-1$
-					nameBuffer.append(this.wildcard.nullAnnotatedReadableName(options, shortNames));
+					nameBuffer.append(this.wildcard.withoutToplevelNullAnnotation().nullAnnotatedReadableName(options, shortNames));
 				} else if (this.lowerBound != null) {
 					nameBuffer.append(" super "); //$NON-NLS-1$
 					nameBuffer.append(this.lowerBound.nullAnnotatedReadableName(options, shortNames));
@@ -316,11 +363,94 @@ public class CaptureBinding extends TypeVariableBinding {
 	    return readableName;
 	}
 
+	@Override
+	public TypeBinding withoutToplevelNullAnnotation() {
+		if (!hasNullTypeAnnotations())
+			return this;
+		if (this.wildcard != null && this.wildcard.hasNullTypeAnnotations()) {
+			WildcardBinding newWildcard = (WildcardBinding) this.wildcard.withoutToplevelNullAnnotation();
+			if (newWildcard != this.wildcard) { //$IDENTITY-COMPARISON$	
+				
+				CaptureBinding newCapture = (CaptureBinding) this.environment.getUnannotatedType(this).clone(null);
+				if (newWildcard.hasTypeAnnotations())
+					newCapture.tagBits |= TagBits.HasTypeAnnotations;
+				newCapture.wildcard = newWildcard;
+				
+				// manually transfer the following two, because we are not in a context where we can call initializeBounds():
+				newCapture.superclass = this.superclass;
+				newCapture.superInterfaces = this.superInterfaces;
+
+				AnnotationBinding[] newAnnotations = this.environment.filterNullTypeAnnotations(this.typeAnnotations);
+				return this.environment.createAnnotatedType(newCapture, newAnnotations);
+			}
+		}
+		return super.withoutToplevelNullAnnotation();
+	}
+
+	@Override
+	TypeBinding substituteInferenceVariable(InferenceVariable var, TypeBinding substituteType) {
+		if (this.pendingSubstitute != null)
+			return this.pendingSubstitute;
+		try {
+			TypeBinding substitutedWildcard = this.wildcard.substituteInferenceVariable(var, substituteType);
+			if (substitutedWildcard != this.wildcard) {  //$IDENTITY-COMPARISON$
+				CaptureBinding substitute = (CaptureBinding) clone(enclosingType());
+			    substitute.wildcard = (WildcardBinding) substitutedWildcard;
+			    this.pendingSubstitute = substitute;
+			    if (this.lowerBound != null)
+			    	substitute.lowerBound = this.lowerBound.substituteInferenceVariable(var, substituteType);
+			    if (this.firstBound != null)
+			    	substitute.firstBound = this.firstBound.substituteInferenceVariable(var, substituteType);
+			    if (this.superclass != null)
+			    	substitute.superclass = (ReferenceBinding) this.superclass.substituteInferenceVariable(var, substituteType);
+			    if (this.superInterfaces != null) {
+			    	int length = this.superInterfaces.length;
+			    	substitute.superInterfaces = new ReferenceBinding[length];
+			    	for (int i = 0; i < length; i++)
+			    		substitute.superInterfaces[i] = (ReferenceBinding) this.superInterfaces[i].substituteInferenceVariable(var, substituteType);
+			    }
+			    return substitute;
+			}
+			return this;
+		} finally {
+			this.pendingSubstitute = null;
+		}
+	}
+	
+	@Override
+	public void setTypeAnnotations(AnnotationBinding[] annotations, boolean evalNullAnnotations) {
+		super.setTypeAnnotations(annotations, evalNullAnnotations);
+		if (annotations != Binding.NO_ANNOTATIONS && this.wildcard != null) {
+			// keep annotations in sync, propagate from capture to its wildcard:
+			this.wildcard = (WildcardBinding) this.wildcard.environment.createAnnotatedType(this.wildcard, annotations);
+		}
+	}
+
 	@Override
 	public TypeBinding uncapture(Scope scope) {
 		return this.wildcard;
 	}
 
+	/*
+	 * CaptureBinding needs even more propagation, because we are creating a naked type
+	 * (during CaptureBinding(WildcardBinding,ReferenceBinding,int,int,ASTNode,int)
+	 * that has no firstBound / superclass / superInterfaces set.
+	 */
+	@Override
+	protected TypeBinding[] getDerivedTypesForDeferredInitialization() {
+		TypeBinding[] derived = this.environment.typeSystem.getDerivedTypes(this);
+		if (derived.length > 0) {
+			int count = 0;
+			for (int i = 0; i < derived.length; i++) {
+				if (derived[i] != null && derived[i].id == this.id)
+					derived[count++] = derived[i];
+			}
+			if (count < derived.length)
+				System.arraycopy(derived, 0, derived = new TypeBinding[count], 0, count);
+		}
+		return derived;
+	}
+
 	public String toString() {
 		if (this.wildcard != null) {
 			StringBuffer buffer = new StringBuffer(10);
