@@ -27,7 +27,7 @@ public class VoidTypeBinding extends BaseTypeBinding {
 		return; // reject misguided attempt.
 	}
 	
-	public TypeBinding unannotated(boolean removeOnlyNullAnnotations) {
+	public TypeBinding unannotated() {
 		return this;
 	}
 }
