@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2013 IBM Corporation and others.
+ * Copyright (c) 2000, 2015 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -7,7 +7,9 @@
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
- *     Stephan Herrmann <stephan@cs.tu-berlin.de> - Contribution for bug 185682 - Increment/decrement operators mark local variables as read
+ *     Stephan Herrmann - Contributions for
+ *								Bug 185682 - Increment/decrement operators mark local variables as read
+ *								Bug 458396 - NPE in CodeStream.invoke()
  *******************************************************************************/
 package org.aspectj.org.eclipse.jdt.internal.eval;
 
@@ -473,7 +475,7 @@ public TypeBinding getOtherFieldBindings(BlockScope scope) {
 	TypeBinding type = ((VariableBinding) this.binding).type;
 	int index = this.indexOfFirstFieldBinding;
 	if (index == length) { //	restrictiveFlag == FIELD
-		this.constant = ((FieldBinding) this.binding).constant();
+		this.constant = ((FieldBinding) this.binding).constant(scope);
 		return type;
 	}
 
@@ -482,7 +484,7 @@ public TypeBinding getOtherFieldBindings(BlockScope scope) {
 	this.otherBindings = new FieldBinding[otherBindingsLength];
 
 	// fill the first constant (the one of the binding)
-	this.constant =((VariableBinding) this.binding).constant();
+	this.constant =((VariableBinding) this.binding).constant(scope);
 
 	// iteration on each field
 	while (index < length) {
@@ -517,7 +519,7 @@ public TypeBinding getOtherFieldBindings(BlockScope scope) {
 			}
 			// constant propagation can only be performed as long as the previous one is a constant too.
 			if (this.constant != Constant.NotAConstant){
-				this.constant = field.constant();
+				this.constant = field.constant(scope);
 			}
 			type = field.type;
 			index++;
